from cocos.director import director
from cocos.layer import Layer, ColorLayer
from cocos.actions import *
from cocos.scene import Scene
from cocos.sprite import *

import pyglet
from pyglet import gl, font
from pyglet.window import key

import soundex

import random

class CreditsLayer(Layer):

    is_event_handler = True

    def __init__(self):
        super(CreditsLayer, self).__init__()

        self.selected = soundex.load('move.mp3')
        self.cheer = soundex.load('cheer.ogg')
        soundex.set_music('credits.mp3')
        self.snapshots = [soundex.load('snap2.ogg'), soundex.load('snap1.ogg')]
        soundex.play_music()

        self.creditList = [name.rstrip() for name in open('CREDITS').readlines()]

        self.text_name = None
        self.text_p = 0
        self.unsay()

        #sprite for flashes.
        self.flash = Sprite( 'fire.png')
        self.flash.visible = False
        self.add(self.flash)

        #PressKey text.
        self.text_pressKey = pyglet.text.Label("Press q to go back",
            font_size=16,
            x=director.get_window_size()[0] /2,
            y=20,
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.CENTER)

    def on_enter(self):
        super(CreditsLayer, self).on_enter()
        random.shuffle( self.creditList )
        self.do( Delay(1) + CallFunc(self.say_next) )

    def on_exit(self):
        super(CreditsLayer, self).on_exit()
        soundex.stop_music()

    def say_next(self):
        if self.text_p >= len(self.creditList):
            soundex.stop_music()
            director.pop()
            return


        self.say(self.creditList[self.text_p] )

        #Start cheers
        if soundex.have_avbin:
            self.cheer.play().volume *= 0.5

        #Wait for cheers and show the new name
        self.do(
            Delay( 13 ) + CallFunc(self.unsay) +
            Delay(0.5) + CallFunc(self.say_next)
            )

        all = 0
        duration = 13
        while all < duration:
            all += random.random()*1.3 + 0.2
            self.do( Delay(all) + CallFunc(self.takeSnapshot) )

        self.text_p += 1

    def takeSnapshot(self):
        #Snapshots are only taken from the middle of the screen to the top.
        x = random.randint(10, director.get_window_size()[0])
        y = random.randint(director.get_window_size()[1]/2, director.get_window_size()[1])
        self.flash.position = x,y
        self.flash.visible = True

        #Since there are only two sounds, x%2 is used to get a 0 or a 1
        #It will be used to:
        # - Chose which sound to play
        #"Audio normalization"
        #Scale of the particule

        dif = x%2
        self.flash.do( ScaleTo( 5 + dif, 0.1+(dif/100.) ) + ScaleTo(0, 0) )
        self.snapshots[dif].play().volume *= (.15 + dif/2.)


    def cheerSound(self):
        if soundex.have_avbin:
            self.cheer.play()

    def draw(self):
        self.text_pressKey.draw()
        if self.text_name:
            self.text_name.draw()

    def unsay(self):
        self.text_name = None

    def say(self, text):
        self.text_name = pyglet.text.Label(text,
            font_size=16,
            x=director.get_window_size()[0] /2,
            y=director.get_window_size()[1] /2,
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.CENTER)


    def on_key_press(self, k, mod):

        if k == pyglet.window.key.Q:
            self.selected.play()
            director.pop()
