import pyglet, time
from pyglet.gl import *

from cocos.layer import *
from cocos.text import *
from cocos.actions import *

class BannerLayer( Layer ):
    def show_message( self, msgs ):

        w,h = director.get_window_size()

        self.banners = []
        for msg in msgs:
            banner = Label( msg, font_size=24,
                font_name='Rehotalko',
                anchor_y='center', anchor_x='left', color=(255,050,050,255))
            banner.position=(-1000, h-200)
            self.add(banner)
            self.banners.append(banner)

        self.ind = 0
        self.next()

    def next(self):
        banner = self.banners[self.ind]
        actions = Accelerate(Delay(2) +
                    MoveBy( (1050,0), duration=1)) + \
                    Delay(1.5) +  \
                    Hide()
        self.ind += 1
        if self.ind < len(self.banners):
            actions += CallFunc(self.next)
        banner.do( actions )

class MessageLayer( Layer ):
    _time_guard = 0
    def show_message(self, msgs, callback=None, timeout=3):
        if time.time() < self._time_guard:
            return
        self._time_guard = time.time() + timeout

        w,h = director.get_window_size()

        for i,msg in enumerate(msgs):
            self.msg = Label( msg,
                font_size=24,
                font_name='Rehotalko',
                anchor_y='center',
                anchor_x='center',
                color=(0,0,255,255))
            self.msg.position=(w/2.0, h-200-i*50)

            self.add( self.msg )

            actions =   Delay(timeout) + Hide()

            if callback:
                actions += CallFunc( callback )

            self.msg.do( ScaleTo(2, timeout) )
            self.msg.do( actions )



class BackgroundLayer2( Layer ):
    def __init__(self):
        super( BackgroundLayer2, self ).__init__()
##         self.img = pyglet.resource.image('background.png')
        self.img = pyglet.resource.image('fondo.png')
        self.position = -100,-100

    def draw( self ):
        glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        glPopMatrix()

class BackgroundLayerPelea( Layer ):
    def __init__(self):
        super( BackgroundLayerPelea, self ).__init__()
        self.img = pyglet.resource.image('fondo-entrenar.png')
        self.position = -150,-100
        self.scale = 0.7

    def draw( self ):
        glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        glPopMatrix()

class BackgroundLayerMainMenu( Layer ):
    def __init__(self):
        super( BackgroundLayerMainMenu, self ).__init__()
##         self.img = pyglet.resource.image('background.png')
        self.img = pyglet.resource.image('main_bg.png')
        self.position = -150,-100
        self.scale = 0.7

    def draw( self ):
        glPushMatrix()
        self.transform()
        self.img.blit(0,0)
        glPopMatrix()

class BackgroundLayerFight( BackgroundLayerMainMenu ):
    def __init__(self):
        super( BackgroundLayerFight, self ).__init__()
##         self.img = pyglet.resource.image('background.png')
        self.img = pyglet.resource.image('fight_bg.png')
        self.position = 0,0
        self.scale = 1

BackgroundLayer=BackgroundLayerMainMenu


class HUD( Layer ):
    def __init__( self ):
        super( HUD, self).__init__()
        self.add( MessageLayer(), name='msg' )
        self.add( BannerLayer(), name='banner' )

    def show_message(self, *args, **kwargs):
        self.get('msg').show_message(*args, **kwargs)

    def show_banner( self, msg):
        self.get('banner').show_message( msg )


