import random, time, shelve

from pyglet.gl import *

from cocos.director import director
from cocos.layer import *
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.actions import *
from particleSystem import *
from cocos.sprite import Sprite
import cocos
from cocos import draw

from constants import *
from actions import *

import world
import player

from sound import Sounds

from gamectrl import GameCtrl

from HUD import HUD, BackgroundLayer2

import animator
from human_skeleton import skeleton as human_skeleton
#from devil import skin as gaucho_skin

## import gaucho
## import gaucho1
## import gaucho2
## import gaucho3
## import gaucho4
## import gaucho5
## import devil

from skeleton import Animate

import copy


from gaucho import skin as padawan_skin


__all__ = ['get_newgame']

def flatten( l ):
    res = []
    for e in l:
        res += list(e)
    return res
import math

class BloodSplat(cocos.cocosnode.CocosNode):
    def splat(self, x, y, direction):
        if direction > 0:
            im = pyglet.resource.image("blood.png")
        else:
            im = pyglet.resource.image("blood.png", flip_x=True)
        sp = Sprite(im)
        sp.scale = 0.1
        angle = random.random()*60-30
        sp.rotation = angle
        ydelta = 20*math.sin(math.radians(angle))
        sp.position = x,y+random.random()*100-50
        sp.do( MoveBy((20*direction,+5+ydelta), 0.2)
             + MoveBy((20*direction,-5+ydelta), 0.2))
        sp.do( ScaleTo(0.8, 0.4) )
        sp.do( Delay(0.2) + FadeTo(0, 0.2) )
        sp.do( Delay(0.6) + CallFunc(lambda: self.remove(sp)) )
        self.add(sp)

def get_newgame(game_settings):
    '''returns the game scene'''
    scene = Scene()

##     r_char = players_settings[R_PLAYER]['char']
##     r_mode = players_settings[R_PLAYER]['mode']
    # model
    left_player_state = player.PlayerState(0, CELLS, NO_DAMAGE, PLAYER_LIFE)
    right_player_state = player.PlayerState(CELLS-1, CELLS, NO_DAMAGE, PLAYER_LIFE)
    #left_player = player.PlayerRL(left_player_state)

    if game_settings['game_mode'] == TRAIN:
#        print "------------------------------------------------------------TRAIN"
        left_player = player.PlayerHuman(initial_state=left_player_state)
    elif game_settings['game_mode'] in (FIGHT,):
        if game_settings['opponent'].script:
            if game_settings['opponent'].script == "alecufacu":
                left_player = player.AlecuFacuPlayer(initial_state=left_player_state,
                                                   char=game_settings['opponent'])
            elif game_settings['opponent'].script == 'goforit':
                left_player = player.GoForItPlayer(initial_state=left_player_state,
                                                   char=game_settings['opponent'])
            elif game_settings['opponent'].script == 'pepelepu':
                left_player = player.PepeLePuPlayer(initial_state=left_player_state,
                                                   char=game_settings['opponent'])
            else:
                left_player = player.PlayerRL(initial_state=left_player_state,
                                          #char=game_settings['opponent'], mode=TRAIN)
                                          char=game_settings['opponent'], mode=FIGHT)
        else:
#            print "------------------------------------------------------------FIGHT"
            left_player = player.PlayerRL(initial_state=left_player_state,
                                      #char=game_settings['opponent'], mode=TRAIN)
                                      char=game_settings['opponent'], mode=FIGHT)

    if game_settings['game_mode'] == HUMAN:
        left_player = player.PlayerHuman(initial_state=left_player_state)
        if game_settings['opponent'].script == "alecufacu":
            right_player = player.AlecuFacuPlayer(initial_state=left_player_state,
                                               char=game_settings['opponent'])
        elif game_settings['opponent'].script == 'goforit':
            right_player = player.GoForItPlayer(initial_state=left_player_state,
                                               char=game_settings['opponent'])
        elif game_settings['opponent'].script == 'pepelepu':
            right_player = player.PepeLePuPlayer(initial_state=left_player_state,
                                               char=game_settings['opponent'])
        else:
            right_player = player.PlayerRL(initial_state=left_player_state,
                                      #char=game_settings['opponent'], mode=TRAIN)
                                      char=game_settings['opponent'], mode=FIGHT)
    else:
        right_player = player.PlayerRL(initial_state=right_player_state,
                                       char=PADAWAN_CHAR, mode=game_settings['game_mode'])
        right_player.timescores.start(game_settings['opponent'])

    print "-------------------"
    print game_settings['game_mode']
    print left_player
    print right_player
    players = [left_player, right_player]

    wrld = world.World(players)

    hud = HUD()

    # controller
    ctrl = GameCtrl(players, hud)

    # view
    audio = Sounds()
    view = GameView(hud, wrld, ctrl, audio, game_settings)


    # add controller
    scene.add( ctrl, z=4, name="controller" )

    # add hud
    scene.add( hud, z=3, name="hud" )
    # add Background
    scene.add( BackgroundLayer2(), z=0, name="background" )


    # add view
    scene.add( view, z=2, name="view" )

    return scene

class ParallaxLayer(Layer):
    def __init__(self):
        super(ParallaxLayer, self).__init__()
        self.parallax = []
        xs, ys = director.get_window_size()
        self.transform_anchor = xs/2,50
        self.wind = 0
        self.soft_wind = 0

    def add_parallax(self, node, depth, z):
        self.parallax.append((node, depth))
        self.add(node, z=z)

    def look_at(self, x, scale):
        self.scale = scale
        sc = self.scale
        xs, ys = director.get_window_size()
        new_x = (xs/2)*sc-x*sc
        old_x = self.position[0]
        dx = new_x - old_x
        self.position = new_x, self.position[1]

        for node, depth in self.parallax:
            nnx = node.position[0]-dx+float(dx)/depth
            node.position = nnx, node.position[1]

    def update_wind(self, dt):
        w_scale = 4
        xs,ys = director.get_window_size()
        if random.random() < 0.005: self.wind *= -1
        dt_wind = (-0.5 + random.random()) * dt * w_scale
        self.wind = max(-2, min(2, (self.wind+dt_wind)))
        self.soft_wind = 0.9*self.soft_wind + 0.1*self.wind
        for c in self.get_children():
            if isinstance(c, Cloud):
                cx = c.position[0]+c.speed*dt
                cx = ( (cx+xs/2)%(xs*2) ) - xs/2
                c.position = cx, c.position[1]

        for c in self.get_children():
            if isinstance(c, Pasto):
                nb = self.soft_wind * 15 + (
                    random.random()*10-5 )
                c.bent = 0.1*nb + 0.9*c.bent


class Cloud(Sprite):
    def __init__(self, img_name):
        super(Cloud, self).__init__(img_name)
##         self.speed_mod = 0.5 + random.random()
##         self.speed = 0
        self.speed = 0.5 + random.random()
        self.speed *= 5


class Pasto(cocos.cocosnode.CocosNode):
    def __init__(self, img_name):
        super(Pasto, self).__init__()
        self.image = pyglet.resource.image(img_name)
        self.bent = 0

    def draw(self):
        glPushMatrix()
        self.transform()
        image = self.image
        x = image.width
        y = image.height

        glEnable(image.target)
        glBindTexture(image.target, image.id)
        glPushAttrib(GL_COLOR_BUFFER_BIT)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        # blit img
        points = [
            (0, 0),
            (x, 0),
            (x+self.bent, y),
            (0+self.bent, y)
            ]
        a,b,_,c,d,_,e,f,_,g,h,_ = image.texture.tex_coords
        textures = [ a,b,c,d,e,f,g,h ]
        new_points = flatten(points)

        pyglet.graphics.draw(4, GL_QUADS,
            ("v2f", new_points),
            ("t2f", textures),
            ("c4B", [255,255,255,255]*4),
            )

        glPopAttrib()
        glDisable(image.target)
        #glDisable(GL_BLEND)
        glPopMatrix()


class PlayerSkin(animator.BitmapSkin):

    def __init__(self, skel, skin, alpha, starting_position, player, view):
        skel = copy.copy(human_skeleton)
        super(PlayerSkin,self).__init__(skel, skin, alpha)
        self.starting_position = starting_position
        self.flipped = False
        self.player = player
        self.current_anim = None
        self.view = view
        self.to_flip = False
        self.idle = False

        self.last_strings = self.skeleton.visit_children(
            lambda bone: bone.get_end()[0])
        self.reset()

    def reset(self, died=False, won=False):
        if died:
            self.do(Speed(Animate(self.orientate(get_action_anim(DIE))), 0.5))
            self.view.audio.player_die()
        elif won:
            self.do(Speed(Animate(self.orientate(get_action_anim(WIN))), 0.5))
#            self.do(Repeat(Animate(self.orientate(get_action_anim(WIN)))))
            self.view.audio.win_sound()
        else:
            self.do(MoveTo(self.starting_position, 0))
        #self.view.on_MATCH_END()

    def orientate(self, anim, alt_anim=None):
        if self.flipped:
            if alt_anim:
                return alt_anim.flipped()
            else:
                return anim.flipped()
        else:
            return anim

    def next_position(self):
        return (BORDER_OFFSET + self.player.next_state.position*CELL_SIZE,
                STARTING_Y_POS)

    def flip(self):
        self.flipped = not self.flipped
        self.to_flip = True

    def do(self, *args, **kwargs):
        super(PlayerSkin, self).do(*args, **kwargs)
        if self.to_flip:
            super(PlayerSkin, self).flip()
            self.to_flip = False

    def update_position(self):
        self.do(MoveTo(self.next_position(), ROUND_DURATION))

    def face(self, other):
        if self.player.state.position < other.state.position:
            if self.flipped:
                self.flip()
            if not other.skin.flipped:
                other.skin.flip()
        elif other.state.position > self.player.state.position:
            if not self.flipped:
                self.flip()
            if other.skin.flipped:
                other.skin.flip()

    def set_animation(self, anim, alt_anim=None):
        self.current_anim = self.orientate(anim, alt_anim)

    def draw(self):
        self.skeleton.propagate_matrix()
        glPushMatrix()
        self.transform()

        bones = self.skeleton.visit_children(
            lambda bone: bone.get_end())
        glColor4ub(159,102,2,95)
        glLineWidth(2)
        for (x,y), ls in zip(bones, self.last_strings):
            glBegin(GL_LINES)
            glVertex2f(x,y)
            glVertex2f(x+ (x-ls)*3,800)
            glEnd()
        glColor4ub(255,255,255,255)
        glPopMatrix()
        self.last_strings = self.skeleton.visit_children(
            lambda bone: bone.get_end()[0])
        super(PlayerSkin, self).draw()


class PowerBar(draw.Canvas):
    health = draw.parameter()

    def __init__(self, player, grow=1):
        super(PowerBar, self).__init__()
        self.player = player
        self.grow = grow
        self.last_energy = None
        self.health = 1
        self.update(0)

    def update(self, dt):
        pe = self.player.state.life
        nsc = pe/float(PLAYER_LIFE)
        if self.health != nsc:
            self.health = nsc

    def render(self):
        xs,ys = director.get_window_size()
        start = xs/2+self.grow*50
        size = 300
        # draw base
        self.set_color((255,255,255,128))
        self.set_stroke_width( 33 )
        self.move_to((start,500))
        self.line_to((start+self.grow*size,500))

        # draw health
        self.set_color((255,0,0,128))
        self.set_stroke_width( 30 )
        self.move_to((start,500))
        self.line_to((start+self.grow*size*self.health,500))
    def xxx(self):
        xs,ys = director.get_window_size()
        pe = self.player.state.life
        nsc = pe/PLAYER_LIFE
        #if self.last_scale is None:
        #    self.sprite.scale = nsc
        #    self.last_scale = nsc
        #else:
        #    if nsc != self.sprite.scale:
        #        self.sprite.do( ScaleTo(nsc, 0.2) )
        #        self.last_scale = nsc
        x = xs/2
        xe = x + self.grow*200
        y = 750
        ye = 700
        self.line.start = x,y
        self.line.end = xe, ye

class StaminaBar(draw.Canvas):
    health = draw.parameter()

    def __init__(self, player, grow=1):
        super(StaminaBar, self).__init__()
        self.player = player
        self.grow = grow
        self.last_energy = None
        self.health = 1
        self.update(0)

    def update(self, dt):
        pe = self.player.state.stamina
        nsc = pe/float(PLAYER_STAMINA)
        if self.health != nsc:
            self.health = nsc

    def render(self):
        xs,ys = director.get_window_size()
        start = xs/2+self.grow*50
        size = 300
        # draw base
        self.set_color((255,255,255,128))
        self.set_stroke_width( 33 )
        self.move_to((start,465))
        self.line_to((start+self.grow*size,465))

        # draw health
        self.set_color((0,0,255,128))
        self.set_stroke_width( 30 )
        self.move_to((start,465))
        self.line_to((start+self.grow*size*self.health,465))
    def xxx(self):
        xs,ys = director.get_window_size()
        pe = self.player.state.stamina
        nsc = pe/PLAYER_STAMINA
        #if self.last_scale is None:
        #    self.sprite.scale = nsc
        #    self.last_scale = nsc
        #else:
        #    if nsc != self.sprite.scale:
        #        self.sprite.do( ScaleTo(nsc, 0.2) )
        #        self.last_scale = nsc
        x = xs/2
        xe = x + self.grow*200
        y = 750
        ye = 700
        self.line.start = x,y
        self.line.end = xe, ye


_firstRun = [True]

class GameView(Layer):

    def __init__(self, hud, world, ctrl, audio, game_settings):
        super(GameView,self).__init__()

        width, height = director.get_window_size()

        self.hud = hud
        self.world = world
        self.ctrl = ctrl
        self.audio = audio
        self.parallax = px = ParallaxLayer()
        self.add( px )


        # names
        if game_settings['opponent']:
            oname = game_settings["opponent"].name
        else:
            oname = "Gauchito Gil"
        t1 = cocos.text.Label(oname, (width/2-40, 530), font_name='Rehotalko',
                              anchor_x="right", anchor_y="center",
                              color=(0,0,0,255))
        self.add(t1, z=10)
        t1 = cocos.text.Label("padawan", (width/2+40, 530), font_name='Rehotalko',
                              anchor_x="left", anchor_y="center",
                              color=(0,0,0,255))
        self.add(t1, z=10)


        # bars
        t1 = cocos.text.Label("health", (width/2, 500), font_name='Rehotalko',
                              anchor_x="center", anchor_y="center",
                              color=(0,0,255,255))
        self.add(t1, z=10)
        t1 = cocos.text.Label("stamina", (width/2, 465), font_name='Rehotalko',
                              anchor_x="center", anchor_y="center",
                              color=(0,0,255,255))
        self.add(t1, z=10)
        self.powerbars = []
        self.staminabars = []

        self.game_settings = game_settings

        if game_settings['opponent']:
            opponent_skin = __import__(game_settings["opponent"].skin).skin
        else:
            opponent_skin = __import__("gaucho1").skin

        if game_settings['game_mode']==HUMAN:
            s1, s2 = opponent_skin, padawan_skin
        else:
            s2, s1 = opponent_skin, padawan_skin
        for pcount, player in enumerate(self.world.players):
            start_pos = (STARTING_X_POS[pcount], STARTING_Y_POS)
            if pcount:
                player.skin = PlayerSkin(copy.copy(human_skeleton), s1, 255, start_pos, player, self)
                player.skin.flip()
                pb = PowerBar(player, 1)
                sb = StaminaBar(player, 1)
            else:
                player.skin = PlayerSkin(copy.copy(human_skeleton), s2, 255, start_pos, player, self)
                pb = PowerBar(player, -1)
                sb = StaminaBar(player, -1)
            self.powerbars.append( pb.update )
            self.staminabars.append( sb.update )
            self.add( pb, z=10 )
            self.add( sb, z=10 )
            px.add(player.skin)
        # bloodsplatter
        self.blood = BloodSplat()
        px.add( self.blood, z=20 )

        self.anim_translate = False

        self.schedule_interval(self.update_world, ROUND_DURATION)

        self.hud.show_message(['Fight!'])
        if _firstRun and game_settings["game_mode"] == TRAIN:
            self.hud.show_banner([
                "Press 'a' to protect low",
                "Press 's' to punch low",
                "Press 'd' to kick low",
                "Press 'q' to protect high",
                "Press 'w' to punch high",
                "Press 'e' to kick high",
                "Use left/right arrows to move",
                "Use up with left/right arrow to jump",
            ])
            _firstRun.pop()

        # pasto front
        pastos = "un_pasto.png", "un_pasto2.png"
        for i in range(12):
            p = Pasto("un_pasto2.png")
            px.add_parallax( p, 0.7, z=3)
            p.position = 1-300+i*600/5,  -75+random.random()*40
        # pasto back
        for i in range(12):
            p = Pasto("un_pasto.png")
            px.add_parallax( p, 1.3, z=-0.6)
            p.position = -300+i*600/5+3, -55+ random.random()*40+60
        # pasto back2
        for i in range(12):
            p = Pasto(pastos[i%2])
            px.add_parallax( p, 1.15, z=-0.5)
            p.position = -300+i*600/5+3, -85+ random.random()*40+60
        # arboles
        arboles = "arbol1.png", "arbol2.png"
        for i in range(5):
            p = Sprite(arboles[i%2])
            px.add_parallax( p, 2, z=-0.7)
            p.position = -100+i*1100/5+3, +40 + random.random()*50+60+100
            p.scale = 0.6
        # nubes
        nubes = "nube1.png", "nube2.png", "nube3.png"
        for i in range(5):
            p = Cloud(nubes[i%3])
            px.add( p,  z=-0.9)
            p.position = -300+i*900/3+3, +70 + random.random()*80+60+300
            p.scale = 1

        if False:
            # add the sun
            s = Sun()
            s.scale = 4
            x = director.get_window_size()[0] - 50
            y = director.get_window_size()[1] - 00
            s.position = x,y
            px.add_parallax( s, 5, z=-10 )

        x,y = director.get_window_size()
        self.transform_anchor = x/2,100
        self.schedule( self.update_view )
        self.schedule( self.parallax.update_wind )

    def update_view(self, dt):
        x,y = director.get_window_size()
        dist = abs(self.world.players[0].skin.position[0] - self.world.players[1].skin.position[0])
        if dist == 0:
            return
        center = self.world.players[0].skin.position[0] + self.world.players[1].skin.position[0]
        center = center / 2
#        scale = min(1.5, 0.5 + 130.0/dist)
        scale = min(1.5, 0.5 + WIN_WIDTH*130.0/640/dist)

        self.parallax.look_at(center, scale)



    def on_enter(self):
        super(GameView,self).on_enter()
        self.audio.fightMusic(True)
        self.audio.match_start()

    def on_exit(self):
        super(GameView,self).on_exit()
        self.audio.fightMusic(False)

    def on_MATCH_END(self):
#        print "MATCH END!"
        humans = [p for p in self.world.players if isinstance(p, player.PlayerHuman)]
        padawan = self.world.players[PADAWAN]
        if not humans:
            # automatic fighting!
            padawan.timescores.end(not padawan.is_dead())
            if padawan.is_dead():
                msg = ["The padawan lost!", "You should train him better!"]
            else:
                msg = ["The padawan defeated your enemy", "Buy him a beer!"]
        else:
            # training session
            human = humans[0]
            if human.is_dead():
                msg = ["The padawan beat you!", "Maybe it's time to make him fight?"]
            else:
                msg = ["You won this round!", "Keep on training the padawan!"]
            # automatically save brain
            padawan.store_brain()
        self.hud.show_message(msg, timeout=10)
        self.unschedule(self.update_world)

## player events
    def _attack_or_tired(self, player, attack_action):
        if player.state.stamina == 0:
            return get_action_anim(TIRED)
        else:
            return get_action_anim(attack_action)


    def on_IDLE(self, player):
        player.skin.set_animation(get_action_anim(IDLE))
        if player.skin.idle == 5:
            self.audio.idle_sound()

    def on_MOVE_LEFT(self, player):
        if not player.collided:
            player.skin.set_animation(get_action_anim(MOVE_BACK),
                                      get_action_anim(MOVE_BACK))
#        self.audio.player_step()
    on_MOVE_BACK = on_MOVE_LEFT
    def on_MOVE_RIGHT(self, player):

        if not player.collided:
            player.skin.set_animation(get_action_anim(MOVE_FORWARD),
                                      get_action_anim(MOVE_FORWARD))
#        self.audio.player_step()
    on_MOVE_FORWARD = on_MOVE_RIGHT
    def on_CROUCH(self, player):
        return

    def on_JUMP(self, player):
        player.skin.set_animation(get_action_anim(JUMP))

    def on_JUMP_LEFT(self, player):
        if player.collided == "border":
            return
        else:
            self.on_JUMP(player)

    def on_JUMP_RIGHT(self, player):
        if player.collided == "border":
            return
        else:
            self.on_JUMP(player)

    def on_DEFEND(self, player):
        player.skin.set_animation(get_action_anim(DEFEND))
        self.audio.player_defend()

    def on_DEFEND_HI(self, player):
        player.skin.set_animation(get_action_anim(DEFEND_HI))
        self.audio.player_defend()

    def on_PUNCH(self, player):
        player.skin.set_animation(self._attack_or_tired(player, PUNCH))
        self.audio.player_punch()

    def on_PUNCH_HI(self, player):
        player.skin.set_animation(self._attack_or_tired(player, PUNCH_HI))
#        player.skin.set_animation(get_action_anim(PUNCH_HI))
        self.audio.player_punch()

    def on_KICK(self, player):
        player.skin.set_animation(self._attack_or_tired(player, KICK))
        self.audio.player_kick()

    def on_KICK_HI(self, player):
        player.skin.set_animation(self._attack_or_tired(player, KICK_HI))
#        player.skin.set_animation(get_action_anim(KICK_HI))
        self.audio.player_kick()
        return

    def on_POWER_BLOW(self, player):
        return

    def on_TURN_AROUND(self, player):
        return

    def on_CONTINUE(self, player):
        if player.action_round >= player.previous_action_duration:
            player.reset_previous_action()


    def on_DAMAGED(self, player):
        #print "PLAYER:", player.__class__
        o = self.world.get_other_than(player)
        if o.state.position < player.state.position:
            d = 1
        else:
            d = -1
        self.blood.splat((1+player.state.position)*CELL_SIZE, 130, d)
        if player.is_dead():
            self.audio.player_die()
        else:
            self.audio.player_hurt()


    def update_hud(self):
        for up in self.powerbars:
            up(0)
        for up in self.staminabars:
            up(0)

    def update_world(self, dt):
        self.ctrl.select_action()

        self.world.world_step()

        for player in self.world.players:

            if player.next_action:
                next_action = player.next_action
            else:
                next_action = IDLE

            action_name = actions_specs[next_action]['name']
#            print "action: ", action_name

            player.skin.face(self.world.get_other_than(player))
            handler = getattr(self, 'on_%s' % action_name)

            if next_action != IDLE:
                player.skin.idle = False

            handler(player)
            if hasattr(player.skin, 'current_anim') and player.skin.current_anim:

#                continuing_action = player.state.idle != idle

##                 if actions_specs[next_action]['rounds'] > 1 and player.next_state.idle:

                print "IDLE: ",  self.world.players[L_PLAYER].state.idle, player.state.position, player.state.jumping

                if player.state.idle:
##                 if next_action != CONTINUE:
                    if next_action != IDLE or player.skin.idle == 5:
                        player.skin.do( Speed(Animate( player.skin.current_anim ,
                                                       recenter_x=self.anim_translate ),
                                              ANIM_SPEED) )

            if next_action == IDLE:
                if player.skin.idle:
                    player.skin.idle -= 1
                else:
                    player.skin.idle = 5

            if player.next_state.position != player.state.position:
                player.skin.update_position()

            if player.next_state.damage_received:
                self.on_DAMAGED(player)


        match_ended = self.world.finish_round()
        if match_ended:
            for player in self.world.players:
                if player.is_dead():
                    player.skin.reset(died=True)
                else:
                    player.skin.reset(won=True)
            self.on_MATCH_END()
        self.update_hud()


