'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

from cocos.director import director
from cocos.layer import *
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.actions import *
from cocos.sprite import *
from cocos.menu import *
from cocos.text import *

import pyglet
from pyglet import gl, font
from pyglet.window import key

from HUD import BackgroundLayerPelea, BackgroundLayerMainMenu, HUD
from HUD import BackgroundLayerFight, BackgroundLayerMainMenu, HUD
import soundex

from Credits import CreditsLayer
from Intro import storyBoard4

from constants import *
import player
import random, os


game_settings = {'game_mode': TRAIN,
                 'opponent': None}

class ScoresLabel( Layer ):
    def __init__(self, text1=None, text2=None):
        super(ScoresLabel, self).__init__()
        if text1 is not None:
            msg = Label(text1, font_size=24, font_name='Rehotalko',
                    anchor_y='center', anchor_x='left', color=(0,0,0,255) )
            msg.position=(30, 60)
            self.add(msg)
        if text2 is not None:
            msg = Label(text2, font_size=24, font_name='Rehotalko',
                    anchor_y='center', anchor_x='left', color=(0,0,0,255) )
            msg.position=(30, 30)
            self.add(msg)

class FightMenu(Menu):
    def __init__(self, scene, who=None):
        self.competition_who = who
        if who == "yourself":
            tit = "Fight yourself vs..."
        else:
            tit = "Padowan vs..."
        super(FightMenu, self).__init__(tit)
        self.select_sound = soundex.load('move.mp3')
        self.scene = scene

        self.font_title['font_name'] = 'Flat Earth Scribe'
        self.font_title['font_size'] = 35
        self.font_title['color'] = (204,30,0,255)

        self.font_item['font_name'] = 'Flat Earth Scribe',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Flat Earth Scribe'
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = 50

        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER
        self.menu_halign = "left"
        self.menu_valign = TOP

        self.padawan = player.PlayerRL(char=PADAWAN_CHAR)
        self.opponent = None
        self.selected_opponent(i=0, removeit=False)
        self.lastSelectedOption = [self.selected_opponent, 0]

        items = []

        for i in range(5):
            items.append( MenuItem('%s' % OPPONENTS[i].name,
                            (lambda x: lambda: self.on_fight(OPPONENTS[x]))(i),
                            self.selected_opponent, i) )

        items.append( MenuItem('Back', self.on_quit, self.selected_back) )
        self.create_menu( items, shake(), shake_back() )

    def on_enter(self):
        super(FightMenu, self).on_enter()
        if self.lastSelectedOption is not None:
            func = self.lastSelectedOption[0]
            args = self.lastSelectedOption[1:]
            func(*args)

    def selected_back(self):
        self.lastSelectedOption = [self.selected_back]
        self.scene.remove("scoresLabel")
        self.scene.add(ScoresLabel(), name="scoresLabel")
        return

    def selected_opponent(self, i, removeit=True):
        if removeit:
            self.scene.remove("scoresLabel")
        self.lastSelectedOption = [self.selected_opponent, i]
        opp = OPPONENTS[i].name
        allscores = self.padawan.timescores.timescores
        if opp in allscores:
            score = "%.1f" % allscores[opp]
        else:
            score = "None"
        self.scene.add(ScoresLabel("Record time:", score), name="scoresLabel")

    def on_fight(self, opponent):
        if self.competition_who == "padowan":
            game_settings['game_mode'] = FIGHT
            game_settings['opponent'] = opponent
        else:
            game_settings['game_mode'] = HUMAN
            game_settings['opponent'] = opponent
        director.push(self._new_game_scene())

    def on_quit( self ):
        director.pop()

    def _new_game_scene(self):
        import gameview
        soundex.stop_music()
        game = gameview.get_newgame(game_settings)
        return game



class GameMenu(Menu):
    def __init__(self, scene):
        super(GameMenu, self).__init__('Train or Figth!')
        self.select_sound = soundex.load('move.mp3')
        self.scene = scene

        self.font_title['font_name'] = 'Flat Earth Scribe'
        self.font_title['font_size'] = 35
        self.font_title['color'] = (204,30,0,255)

        self.font_item['font_name'] = 'Flat Earth Scribe',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Flat Earth Scribe'
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = 50

        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER
        self.menu_halign = "left"
        self.menu_valign = TOP

        self.padawan = player.PlayerRL(char=PADAWAN_CHAR)
        self.opponent = None
        self.selected_train(removeit=False)
        self.lastSelectedOption = [self.selected_train]

        items = []

        items.append( MenuItem('Train padowan', self.on_train, self.selected_train) )
        items.append( MenuItem('Padowan vs...', self.on_padowan, self.selected_clear) )
        items.append( MenuItem('Fight yourself vs...', self.on_yourself, self.selected_clear) )
        items.append( MenuItem('Back', self.on_quit, self.selected_clear) )
        self.create_menu( items, shake(), shake_back() )

    def on_enter(self):
        super(GameMenu, self).on_enter()
        if self.lastSelectedOption is not None:
            func = self.lastSelectedOption[0]
            args = self.lastSelectedOption[1:]
            func(*args)

    def selected_train(self, removeit=True):
        self.lastSelectedOption = [self.selected_train]
        if removeit:
            self.scene.remove("scoresLabel")
        trainq = self.padawan.timescores.cant_trainnings
        if trainq == 0:
            txt = "Never trained"
        elif trainq == 1:
            txt = "Trained once"
        elif trainq == 2:
            txt = "Trained twice"
        else:
            txt = "Trained %d times" % trainq
        self.scene.add(ScoresLabel(None, txt), name="scoresLabel")

    def on_train( self ):
        game_settings['opponent'] = self.opponent = None
        game_settings['game_mode'] = TRAIN
        director.push(self._new_game_scene())

    def _new_game_scene(self):
        import gameview
        soundex.stop_music()
        game = gameview.get_newgame(game_settings)
        return game

    def selected_clear(self):
        self.lastSelectedOption = [self.selected_clear]
        self.scene.remove("scoresLabel")
        self.scene.add(ScoresLabel(), name="scoresLabel")
        return

    def on_quit( self ):
        director.pop()

    def on_padowan(self):
        game = Scene()
        game.add(FightMenu(game, "padowan"))
        game.add( BackgroundLayerPelea(), z=0 )
        director.push(game)

    def on_yourself(self):
        game = Scene()
        game.add(FightMenu(game, "yourself"))
        game.add( BackgroundLayerPelea(), z=0 )
        director.push(game)


class OptionsMenu( Menu ):
    def __init__(self):
        super(OptionsMenu, self).__init__('- Options -')
        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = 'Flat Earth Scribe'
        self.font_title['font_size'] = 35
        self.font_title['color'] = (204,30,0,255)

        self.font_item['font_name'] = 'Flat Earth Scribe',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Flat Earth Scribe'
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = 50

        # you can also override the font size and the colors. see menu.py for
        # more info
        self.menu_halign = "left"
        self.menu_valign = TOP
        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER

        items = []

        self.volumes = ['Mute','10','20','30','40','50','60','70','80','90','100']

        items.append( MultipleMenuItem(
                        'SFX volume: ',
                        self.on_sfx_volume,
                        self.volumes,
                        int(soundex.sound_vol * 10) )
                    )
        items.append( MultipleMenuItem(
                        'Music volume: ',
                        self.on_music_volume,
                        self.volumes,
                        int(soundex.music_player.volume * 10) )
                    )
        items.append( ToggleMenuItem('Show FPS:', self.on_show_fps, director.show_FPS) )
        items.append( MenuItem('Fullscreen', self.on_fullscreen) )
        items.append( MenuItem('Back', self.on_quit) )
        self.create_menu( items, shake(), shake_back() )

    def on_fullscreen( self ):
        director.window.set_fullscreen( not director.window.fullscreen )

    def on_quit( self ):
        self.parent.switch_to( 0 )

    def on_show_fps( self, value ):
        director.show_FPS = value

    def on_sfx_volume( self, idx ):
        vol = idx / 10.0
        soundex.sound_volume( vol )
        soundex.play('punch.ogg')

    def on_music_volume( self, idx ):
        vol = idx / 10.0
        soundex.music_volume( vol )

class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__('Gauchito Gil vs. The Devil')

        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        # you can also override the font size and the colors. see menu.py for
        # more info
        self.font_title['font_name'] = 'Flat Earth Scribe'
        self.font_title['font_size'] = 40
        self.font_title['color'] = (204,30,0,255)

        self.font_item['font_name'] = 'Flat Earth Scribe',
        self.font_item['color'] = (32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = 'Flat Earth Scribe'
        self.font_item_selected['color'] = (0,0,0,255)
        self.font_item_selected['font_size'] = 50

        self.menu_halign = "left"
        self.menu_valign = TOP

        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = CENTER
        self.menu_anchor_x = CENTER

        items = []
        items.append( MenuItem('New game', self.on_game_settings))
        items.append( MenuItem('Saved game', self.on_continue) )
        items.append( MenuItem('Concept', self.on_concept) )
        items.append( MenuItem('High scores', self.on_highscores) )
        items.append( MenuItem('Options', self.on_options) )
        items.append( MenuItem('Credits', self.on_credits) )
        items.append( MenuItem('Intro', self.on_intro) )
        items.append( MenuItem('Quit', self.on_quit) )

        self.create_menu( items, shake(), shake_back() )

    def on_enter(self):
        super( MainMenu, self).on_enter()
        soundex.set_music('intro.mp3')
        soundex.play_music()

    def on_options(self):
        self.parent.switch_to(1)

    def on_concept(self):
        game = Scene()
        expl = Scene(GameDinamicExplanationLayer())
        director.push(expl)

    def on_continue(self):
        # continuamos el juego, sin tocar las variables
        game = Scene()
        game.add(GameMenu(game))
        game.add( BackgroundLayerPelea(), z=0 )
        #expl = Scene(GameDinamicExplanationLayer())
        director.push(game)

    def on_game_settings(self):
        # reseteamos todo
        self.padawan = player.PlayerRL(char=PADAWAN_CHAR)
        self.padawan.timescores.reset()
        padawan_brain = "data/Gil.brain"
        if os.access(padawan_brain, os.W_OK):
            os.remove(padawan_brain)

        # continuamos el juego
        game = Scene()
        game.add(GameMenu(game))
        game.add( BackgroundLayerPelea(), z=0 )
        expl = Scene(GameDinamicExplanationLayer())
        director.push( SequenceScene(expl, game))

    def on_highscores(self):
        scene = Scene()
        back = BackgroundLayerMainMenu()
        scene.add(back, z=0)
        white = ColorLayer(255,255,255,210)
        scene.add(white, z=1)
        scene.add(HiScoresLayer(), z=2)
        director.push(scene)

    def on_credits(self):
        director.push( Scene(CreditsLayer()) )

    def on_intro(self):
        soundex.stop_music()
        soundex.set_music('loopintro.ogg')
        soundex.play_music()
        director.push( Scene(storyBoard4()) )

    def on_quit(self):
        soundex.stop_music()
        pyglet.app.exit()


gameDinamic = """
This is not a fighting game.  You, the Master Puppeteer, need to train your padawan.

When the training starts, your padawan does not know how to fight. But he's capable of learning. You can fight unlimited rounds with your padawan, and he'll grow stronger and wiser.

After you train your padawan, you can send him to fight against your enemies. If your padawan loses, you should train him more.

Make him fight with the different enemies, you'll see his strenghts and weaknesses, and you'll be able to train him better.
"""

class GameDinamicExplanationLayer(Layer):
    is_event_handler = True
    def __init__(self):
        super(GameDinamicExplanationLayer, self).__init__()

        x,y = director.get_window_size()

        self.text = RichLabel(font_name="Komika Text", text=gameDinamic,
            font_size=20, x=x/2, y=y/2, anchor_x="center", anchor_y="center",
            width=700, multiline=True)
        self.add( self.text  )

    def on_key_press(self, k, mods):
        if k in (key.ENTER, key.SPACE, key.ESCAPE):
            director.pop()
        return True

class HiScoresLayer(Layer):
    is_event_handler = True
    def __init__(self):
        super(HiScoresLayer, self).__init__()

        self.xs,self.ys = director.get_window_size()

        t = Label(font_name="Flat Earth Scribe", font_size=40,
                    anchor_x="center", anchor_y="center", x=self.xs/2, y=self.ys-50,
                    color=(204,30,0,255), text="High Scores",
                 )
        self.add(t)

        totalscore = self.scores()
        self.ranking(totalscore)

        if totalscore is None:
           text = "To rank with the Masters you need to defeat all the enemies!"
           tam = 15
        else:
           text = "Total score: %d" % totalscore
           tam = 40
        t = Label(font_name="Rehotalko", font_size=tam, text=text,
                    anchor_x="center", anchor_y="center", x=self.xs/2, y=50,
                    color=(0,0,0,255),
                 )
        self.add(t)

    def scores(self):
        self.padawan = player.PlayerRL(char=PADAWAN_CHAR)
        text = "Trainning sessions: %d" % self.padawan.timescores.cant_trainnings
        t = Label(font_name="Rehotalko", font_size=15, text=text,
                    anchor_x="center", anchor_y="center", x=self.xs/2, y=20,
                    color=(0,0,0,255),
                 )
        self.add(t)

        allscores = self.padawan.timescores.timescores
        scores = []
        for opp in OPPONENTS:
            if opp.name in allscores:
                score = "%6.1f" % allscores[opp.name]
            else:
                score = "  None"
            scores.append((opp.name, score))

        maxname = max(len(x[0]) for x in scores)
        text = "Times".center(maxname+8)
        t = Label(font_name="SV Basic Manual", font_size=20, text=text,
                    anchor_x="left", anchor_y="center", x=30, y=self.ys-150,
                    color=(50,50,255,255),
                 )
        self.add(t)
        for i,(name,score) in enumerate(scores):
            text = "%-*s  %s" % (maxname, name, score)
            t = Label(font_name="SV Basic Manual", font_size=20, text=text,
                        anchor_x="left", anchor_y="center", x=30, y=self.ys-200-i*50,
                        color=(50,50,255,255),
                     )
            self.add(t)

        values = allscores.values()
        if None in values or len(values) != len(OPPONENTS):
            return None
        tot = 50000 / sum(x**2 for x in values)
        return tot

    def ranking(self, tot=None):
        autores = [x.strip() for x in open("CREDITS")]
        random.shuffle(autores)
        info = zip(HISCORES, autores)
        if tot is not None:
            info += [(int(tot), "Padawan")]
        info.sort(reverse=True)

        maxname = max(len(x[1]) for x in info)
        text = "Scores".center(maxname+6)
        t = Label(font_name="SV Basic Manual", font_size=20, text=text,
                    anchor_x="left", anchor_y="center", x=350, y=self.ys-150,
                    color=(255,50,50,255),
                 )
        self.add(t)
        for i,(score, autor) in enumerate(info):
            text = "%-*s  %4d" % (maxname, autor, score)
            t = Label(font_name="SV Basic Manual", font_size=20, text=text,
                        anchor_x="left", anchor_y="center", x=350, y=self.ys-200-i*35,
                        color=(255,50,50,255),
                     )
            self.add(t)
        return

    def on_key_press(self, k, mods):
        if k in (key.ENTER, key.SPACE, key.ESCAPE):
            director.pop()
        return True

class errorLayer(Layer):

    is_event_handler = True     #: enable pyglet's events

    def __init__( self, errorText ):

        super(errorLayer, self).__init__()

        self.text_title = pyglet.text.Label("ERROR",
            font_size=32,
            x=director.get_window_size()[0] /2,
            y=director.get_window_size()[1],
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.TOP )

        self.text_help = pyglet.text.Label( errorText,
            font_size=12,
            x=director.get_window_size()[0] /2,
            y=director.get_window_size()[1] /2 - 90,
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.CENTER)

        self.text_pressKey = pyglet.text.Label("Press ENTER to continue",
            font_size=16,
            x=director.get_window_size()[0] /2,
            y=20,
            anchor_x=font.Text.CENTER,
            anchor_y=font.Text.CENTER)

    def draw( self ):
        self.text_title.draw()
        self.text_help.draw()
        self.text_pressKey.draw()

    def on_key_press( self, k , m ):
        if k == key.ENTER:
            director.pop()

class grossini_silent(Layer):
    def __init__( self ):
        super( grossini_silent, self ).__init__()

        g = Sprite( 'grossini_silent.png')
        g.position = (320,240)

        self.add( g )

def errorScene( errorText ):
    scene = Scene()
    scene.add(errorLayer(errorText), z=0)

    return scene

def mainMenuScene():
    scene = Scene()
    scene.add( MultiplexLayer(
                    MainMenu(),
                    OptionsMenu(),
                    ),
                z=1 )
    scene.add( BackgroundLayerMainMenu(), z=0 )

    return scene



class FBLayer(Layer):
    def __init__( self ):
        super( FBLayer, self ).__init__()

        self.text_help = pyglet.text.Label("Some effects won\'t be shown.",
        font_size=16,
        x=320,
        y=260,
        anchor_x=font.Text.CENTER,
        anchor_y=font.Text.CENTER)

        self.text_faq = pyglet.text.Label("Visit http://www.cocos2d.org/doc/faq/",
        font_size=16,
        x=320,
        y=240,
        anchor_x=font.Text.CENTER,
        anchor_y=font.Text.CENTER)

    def draw( self ):
        self.text_help.draw()
        self.text_faq.draw()


class SequenceScene(Scene):
    def __init__(self, *scenes):
        super(SequenceScene, self).__init__()
        self.scenes = scenes
        self.p = 0

    def on_enter(self):
        if self.p >= len(self.scenes):
            director.pop()
        else:
            director.push( self.scenes[ self.p ] )

        self.p += 1


def main():

    data_path = os.path.abspath('data')
    pyglet.resource.path.append(data_path)
    pyglet.resource.reindex()
    font.add_directory(data_path)
    global HAVE_FRAMEBUFFER
    sceneList = []
    director.init( resizable=True, width=WIN_WIDTH, height=WIN_HEIGHT )

    try:
        import cocos.gl_framebuffer_object as FG
        FG.FramebufferObject().check_status()
        HAVE_FRAMEBUFFER = True
    except Exception:
        HAVE_FRAMEBUFFER = False

    try:
        import pyglet.media.avbin as testAvbin
        HAVE_AVBIN = True
    except:
        HAVE_AVBIN = False

    try:
        import numpy as NP
        HAVE_NUMPY = True
    except:
        HAVE_NUMPY = False

    if not HAVE_NUMPY:
        numpyError = "You need NumPy. Get it from http://numpy.scipy.org/"
        sceneList.append( errorScene(numpyError) )

    if not HAVE_FRAMEBUFFER:
        FBerror = "Your GPU doesn't support Frame Buffers Objects"
        sceneList.append( errorScene( FBerror).add(FBLayer(), z=1) )

    if not HAVE_AVBIN:
        soundError = "Please get avbin from http://code.google.com/p/avbin/"
        sceneList.append( errorScene( soundError).add( grossini_silent(), z=1) )

    soundex.stop_music()
    soundex.set_music('loopintro.ogg')
    soundex.play_music()
    sceneList.append( Scene(storyBoard4()) )
    sceneList.append( mainMenuScene() )

    director.run( SequenceScene(*sceneList) )


if __name__ == "__main__":
    main()
