import copy, sys, os, time, cPickle
import reply
from actions import *
from constants import *

class Player(object):

    def __init__(self, initial_state=None, char=None):
        if initial_state is None:
            self.initial_state = PlayerState()
            self.state = PlayerState()
        else:
            self.initial_state = initial_state
            self.state = self.initial_state

        self.next_state = copy.copy(self.state)
        self.next_action = IDLE

        self.reset_previous_action()
        self.collided = False
        self.skin = None
        self.char = char

    def think(self, other):
        self.next_action = self.get_action(self.state)

    def get_action(self, state):
        return self.next_action

    def is_dead(self, state=None):
        if not state: state = self.state
        dead = False
        if state.life <= 0:
            # player is dead, so remove its remaining stamina
            state.stamina = 0
            state.life = 0
            dead = True
        return dead


    def update_state(self,*args, **kwargs): pass


    def reset(self):
        # reset the current state to its initial value
        self.state = copy.copy(self.initial_state)
        self.next_state = copy.copy(self.state)

    def reset_previous_action(self):
        self.key_action = None
        self.action_round = 0
        self.previous_action = None
        self.previous_action_duration = 0

    def store_brain(self, *args, **kwargs): pass

class AlecuFacuPlayer(Player):
    def think(self, other):
        if abs(self.state.position - other.state.position) > 1:
            self.next_action = MOVE_FORWARD
        if abs(self.state.position - other.state.position) == 1:
            if self.state.stamina > 1:
                self.next_action = PUNCH
            else:
                self.next_action = DEFEND
        if abs(self.state.position - other.state.position) == 0:
            self.next_action = JUMP_RIGHT

class GoForItPlayer(Player):
    def think(self, other):
        if abs(self.state.position - other.state.position) > 1:
            self.next_action = MOVE_FORWARD
        if self.state.stamina > 1:
            self.next_action = random.choice([PUNCH, PUNCH_HI])
        else:
            self.next_action == IDLE

class PepeLePuPlayer(Player):
    def __init__(self, *args, **kwargs):
        super(PepeLePuPlayer, self).__init__(*args, **kwargs)
        self.reset_pending_actions()

    def reset_pending_actions(self):
        self.pending_actions = [PUNCH_HI, PUNCH_HI, DEFEND_HI, MOVE_FORWARD, PUNCH, DEFEND]

    def think(self, other):
        if self.state.stamina > 1:
            if self.pending_actions:
                self.next_action = self.pending_actions.pop()
                if not self.pending_actions:
                    self.reset_pending_actions()
        else:
            self.next_action = random.choice([DEFEND, DEFEND_HI, MOVE_BACK])

class DancePlayer(Player):
    def __init__(self, *args, **kwargs):
        super(DancePlayer, self).__init__(*args, **kwargs)
        self.count = 0

    def think(self, other):
        if abs(self.state.position - other.state.position) > 1:
            self.next_action = MOVE_FORWARD
        if self.state.stamina > 1:
            self.next_action = random.choice([PUNCH, PUNCH_HI])
        else:
            self.next_action == IDLE

class PlayerHuman(Player):

    def __init__(self, initial_state=None, char=None):
        super(PlayerHuman, self).__init__(initial_state, char)
        self.key_action = IDLE

    def think(self, other):
        self.next_action = self.key_action


class TimeScores(object):
    timescores = {}
    _canttrainnings = [0]
    def __init__(self, charname):
        self.competing = None
        self.start_time = None
        self.filename = "data/%s.tscores" % charname
        if os.access(self.filename, os.R_OK):
            f = open(self.filename)
            oldscores, oldtrains = cPickle.load(f)
            f.close()
            self.timescores.clear()
            self.timescores.update(oldscores)
            self._canttrainnings[0] = oldtrains

    def reset(self):
        self._canttrainnings[0] = 0
        self.timescores.clear()
        self._persist()

    cant_trainnings = property(lambda s: s._canttrainnings[0])
    def addTrainningSession(self):
        self._canttrainnings[0] += 1
        self._persist()

    def _persist(self):
        f = open(self.filename, "wb")
        cPickle.dump((self.timescores, self._canttrainnings[0]), f)
        f.close()

    def start(self, opponent):
        if opponent is not None:
            opponent = opponent.name
        self.competing = opponent
        self.start_time = time.time()

    def end(self, won):
        if self.competing is None:
            raise ValueError("Llamamos al TimeScores.end() sin el start!")

        # guardamos la info si gano
        if won:
            elapsed = time.time() - self.start_time
            nuevovalor = min(elapsed, self.timescores.get(self.competing, sys.maxint))
            self.timescores[self.competing] = nuevovalor
            self._persist()

        # reseteamos estado
        self.competing = None
        self.start_time = None

class PlayerRL(Player):

    def __init__(self, initial_state=None, char=None, mode=None):
        super(PlayerRL, self).__init__(initial_state, char)
        space = PlayerRL.get_state_space(), PlayerRL.get_action_space()
        encoder = reply.encoder.DistanceEncoder(*space)
        # define a storage for the brain initialized to 0
        self.storage = reply.storage.TableStorage(encoder, {})
        # constants used for learning
        alpha = 0.9
        epsilon = 0.1
        gamma = 0.9
        alpha_decay = 1.0
        epsilon_decay = 0.999
        # training mode brain
        learner = reply.learner.QLearner(alpha, gamma, alpha_decay)
        #selector = reply.selector.EGreedySelector(epsilon, epsilon_decay)
        temperature = 0.01
        selector = reply.selector.SoftMaxSelector(temperature)
        # fight mode brain
        # this brain does not learn, but just return the best action available
        fighter = reply.learner.QLearner(0.0, 0.0)
        evaluator = reply.selector.SoftMaxSelector(temperature)

        self.brains = {TRAIN: reply.RL(learner, self.storage, encoder, selector),
                       FIGHT: reply.RL(fighter, self.storage, encoder, evaluator)}

        if mode is not None:
            self.set_mode(mode)
        self.load_brain(char)
        self.timescores = TimeScores(char.name)
        if char == PADAWAN_CHAR and mode==TRAIN:
            self.timescores.addTrainningSession()

    def store_brain(self, char=None):
        if char is None:
            char = self.char
        self.storage.dump('data/%s.brain' % char.name)

    def load_brain(self, char):
        try:
            self.storage.load('data/%s.brain' % char.name)
        except:
            pass

    def set_mode(self, mode):
        self.mode = mode
        self.brain = self.brains[self.mode]
        #print 'SET MODE... ', self.brain.storage
        self.brain.new_episode(self)

    def think(self, other):
#        print self.brain.storage.state
#        print '-'*80
        self.state.distance = abs(self.state.position-other.state.position)
        self.next_state.distance = abs(self.next_state.position-other.next_state.position)
        finished = not self.brain.step(self)

        ## Ugly HACK
        if self.next_action == None:
            self.next_action = IDLE


    def update_state(self, opponent):
        opponent.state.distance = abs(self.state.position-opponent.state.position)
        opponent.next_state.distance = abs(self.next_state.position-opponent.next_state.position)
        self.learn(opponent.state, opponent.next_state, opponent.next_action,
                   self.get_reward(opponent.next_state))
        #    pass
        #opponent_reward = self.get_opponent_reward(opponent_next_state)
        #self.learn(self.state, self.next_state, self.next_action, reward)
        # update current state
        super(PlayerRL, self).update_state(opponent)

    def learn(self, state, next_state, action, reward):
        encoded_state = self.brain.encoder.encode_state(state)
        encoded_next_state = self.brain.encoder.encode_state(next_state)

        # perform the learning
        self.brain.learner.update(
            encoded_state,
            action,
            reward,
            encoded_next_state,
        )

    def get_opponent_reward(self, state):
        reward = self.get_reward(state)
        if self.is_dead(state) or \
           state.damage not in [DAMAGE_DONE_AND_RECEIVED, DAMAGE_DONE_AND_RECEIVED_HI]:
            reward = -reward
        return reward



    def reset(self):
        super(PlayerRL, self).reset()
        self.brain.new_episode(self)

    # reply required methods
    @staticmethod
    def get_state_space():
        return [reply.dimension(0, CELLS-1, CELLS),
                reply.dimension(0, PLAYER_STAMINA-1, PLAYER_STAMINA),
                reply.dimension(0,1,2) # idle reward
                ]

##         # [distance, opponent_action]
##         return [reply.dimension(0, 24, 25)]#,
##                 #reply.dimension(0, 3, 4)]

    @staticmethod
    def get_action_space():
#        return [reply.dimension(0, 14, 15)]
        return [reply.dimension(0, 11, 12)]

    def get_reward(self, state):
        return state.damage_done - state.damage_received
#        if self.is_dead(state):
#            print 'YOU ARE DEAD'
##            return -10
#            return 0

##        print 'YOUR STATE is', list(state)
#        if state.damage in [DAMAGE_DONE, DAMAGE_DONE_AND_RECEIVED]:
#            reward = 1
#        else:
#            reward = 0
#        return reward
        if state.damage == NO_DAMAGE:
            reward = 0
        elif state.damage == DAMAGE_DONE_HI:
            reward = 2
        elif state.damage == DAMAGE_DONE:
            reward = 1
        elif state.damage == DAMAGE_RECEIVED_HI:
            reward = -2
        elif state.damage == DAMAGE_RECEIVED:
            reward = -1
        elif state.damage == DAMAGE_DONE_AND_RECEIVED_HI:
            reward = 0.5
        elif state.damage == DAMAGE_DONE_AND_RECEIVED:
            reward = 0.25
##            reward = 1
        # modify reward according to stamina
        if state.stamina == 0:
            reward = 0.25 * reward
        elif state.stamina == 1:
            reward = 0.5 * reward
        elif state.stamina == 2:
            # keep full reward
            pass

#        print 'REWARD:', reward
        return reward

    def do_action(self, solver, action):
##        print 'ACTION TO DO', action
        self.next_action = int(action)

    def get_initial_state(self):
        return self.initial_state

    def get_state(self):
        return self.state

    def is_final(self, state):
        return self.is_dead(state)


class PlayerState(object):

    def __init__(self, position=0, distance=0, damage=0, life=PLAYER_LIFE, stamina=PLAYER_STAMINA):
        self.position = position
        self.distance = distance
        self.damage_received = 0
        self.damage_done = 0
        self.life = life
        self.stamina = stamina
        self.idle = True
        self.jumping = 0
        self.jump_direction = 0

    def __iter__(self):
        #return iter([self.distance, self.opponent_action])
        return iter([self.distance, self.stamina, self.idle])

    def __eq__(self, other):
        equal = self.position == other.position
        equal &= self.distance == other.distance
        equal &= self.damage == other.damage
        equal &= self.life == other.life
        equal &= self.stamina == other.stamina
        return equal


