import copy
import random
from actions import *
from constants import *

class World(object):

    def __init__(self, players):
        self.players = players
        # assign players to this world
        for player in self.players:
            player.world = self

    def get_players_distance(self):
        distance = abs(self.players[0].next_state.position - self.players[1].next_state.position)
        return distance

    def get_other_than(self, player):
        players = list(self.players)
        players.remove(player)
        other = players[0]
        return other


    def world_step(self):
        # make them think
        [ p.think(self.get_other_than(p)) for p in self.players ]
        #[ p.think() for p in self.players ]

        p1, p2 = self.players
        if p1.state.position > p2.state.position:
            left = p2
            right = p1
        else:
            left = p1
            right = p2

        # si se chocan, no avanzan
        if p1.next_action == MOVE_FORWARD == p2.next_action and \
           abs(p1.state.position - p2.state.position) == 1:
            print "CRASH"
            p1.next_action == IDLE
            p2.next_action == IDLE

        # si se chocan, no avanzan
        if p1.next_action == MOVE_FORWARD == p2.next_action and \
           abs(p1.state.position - p2.state.position) == 2:
            if random.random() < 0.5:
                print "p1 crash"
                p1.next_action == IDLE
            else:
                print "p2 crash"
                p2.next_action == IDLE

        # que no se salgan del ring
        if left.next_action == MOVE_BACK and left.state.position == 0:
            left.next_action = IDLE
        if right.next_action == MOVE_BACK and right.state.position == CELLS-1:
            right.next_action = IDLE

        # limitamos salto
        if left.next_action == JUMP_RIGHT and left.state.position >= CELLS-2:
            left.next_action = IDLE
        if left.next_action == JUMP_LEFT and left.state.position <= 1:
            left.next_action = IDLE
        if right.next_action == JUMP_LEFT and right.state.position >= CELLS-2:
            right.next_action = IDLE
        if right.next_action == JUMP_RIGHT and right.state.position <= 1:
            right.next_action = IDLE
        distance = self.get_players_distance()

        for p in self.players:
            # no mover ni nada si en idle

            # para saltos, te paso a idle en jumping pasos
            if not p.state.idle:
                if p.state.jumping:
                    p.next_state.jumping -= 1
                    p.next_state.position += p.state.jump_direction
                    if not p.next_state.jumping:
                        p.next_state.idle = True
                    continue

            if p.state.idle == False: continue


            if p.next_action == MOVE_FORWARD:
                if p == self.players[L_PLAYER]:
                    print p,"move forward"
                if distance != 1:
                    if p is left:
                        p.next_state.position += 1
                    else:
                        p.next_state.position -= 1

            if p.next_action == MOVE_BACK:
                if p == self.players[L_PLAYER]:
                    print p,"move back"
                if p is left:
                    p.next_state.position -= 1
                else:
                    p.next_state.position += 1

            # right is forward
            if p.next_action == JUMP_RIGHT:
                p.next_state.jumping = 2

                p.next_state.idle = False
                if p is left:
                    p.next_state.jump_direction = +1
                else:
                    p.next_state.jump_direction = -1
            # right is forward
            if p.next_action == JUMP_LEFT:
                p.next_state.jumping = 2

                p.next_state.idle = False
                if p is left:
                    p.next_state.jump_direction = -1
                else:
                    p.next_state.jump_direction = +1

        # ahora la violencia
        distance = self.get_players_distance()

        s = p.state.stamina
        smod = 0
        if s==1: smod = 0.3
        if s==2: smod = 0.6
        if s==3: smod = 1

        for p in self.players:
            other = self.get_other_than(p)
            # segundo turno de patadas
            if p.state.idle == False and not p.state.jumping:
                p.next_state.idle = True
                p.next_action = IDLE
                if distance == KICK_DISTANCE:
                    other.next_state.damage_received = 2*smod
                    p.next_state.damage_done = 2*smod
                continue

            if p.state.stamina == 0: continue

            if p.next_action == KICK_HI:
                p.next_state.idle = False
                p.next_state.stamina -= 1

            if p.next_action == KICK:
                p.next_state.idle = False
                p.next_state.stamina -= 1

            for action, counter in [[PUNCH, DEFEND], [PUNCH_HI, DEFEND_HI]]:
                if p.next_action == action:
                    p.next_state.stamina -= 1
                    if distance == PUNCH_DISTANCE:
                        if other.next_action != counter:
                            p.next_state.damage_done += 1*smod
                            other.next_state.damage_received += 1*smod
                        else:
                            p.next_state.damage_done += 0.1
                            other.next_state.damage_received += 0.1


        for p in self.players:
            if p.next_action in (IDLE, DEFEND, DEFEND_HI, None):
                p.next_state.stamina += 1
            else:
                pass
#                print p, p.next_action, "NOT IN", (IDLE, DEFEND, DEFEND_HI)

            p.next_state.stamina = min(PLAYER_STAMINA, max(0, p.next_state.stamina))


    def make_next_state(self, state):
        new_state = copy.copy(state)
        new_state.life -= new_state.damage_received
        new_state.damage_received = new_state.damage_done = 0
        return new_state

    def finish_round(self):
        player = self.players[0]
        other = self.players[1]

        player.update_state(other)
        other.update_state(player)

        for p in self.players:
            p.state = p.next_state
            p.next_state = self.make_next_state(p.state)
            p.next_action = IDLE


        match_ended = player.is_dead() or other.is_dead()
#        for player in self.players:
#            other = self.get_other_than(player)
#            if not player.is_dead():
#                player.update_state(other.next_action)
#            else:
#                match_ended = True
        return match_ended

